<?php
/*
Plugin Name: Availability Calendar
Plugin URI: http://wordpress.org/extend/plugins/availability/
Description: A simple to use and manage availability calendar.
Version: 0.1
Author: Steve Whiteley
Author URI: http://36flavours.com
*/
if (!class_exists('Availability')) {
	class Availability {
		var $name = 'Availability';
		var $tag = 'availability';
		var $db = '0.1';
		var $options = array();
		var $year, $start, $end, $table;
		function Availability()
		{
			global $wpdb;
			$this->table = $wpdb->prefix.$this->tag;
			if ($options = get_option($this->tag)) {
				$this->options = $options;
				$this->start = $this->year = gmdate('Y', current_time('timestamp'));
				$this->end = $this->start + $this->options['years'] - 1;
			}
			if (isset($_GET['y']) && is_numeric($_GET['y'])) {
				if ($_GET['y'] <= $this->start) {
					$this->year = $this->start;
				} else if ($_GET['y'] >= $this->end) {
					$this->year = $this->end;
				} else {
					$this->year = $_GET['y'];
				}
			}
			if (is_admin()) {
				add_action('admin_menu', array(&$this, 'admin_menu'));
				add_filter('plugin_row_meta', array(&$this, 'plugin_row_meta'), 10, 2);
				register_activation_hook(__FILE__, array(&$this, 'activate'));
			} else {
				add_action('get_header', array(&$this, 'css'));
				add_shortcode($this->tag, array(&$this, 'shortcode'));
				add_filter('availability_dropdown', array(&$this, 'dropdown'), 1);
			}			
		}
		function activate()
		{
			if (!isset($this->options['years'])) {
				if (update_option($this->tag, array(
					'years' => 5
				))) {
					$this->options['years'] = 5;
				}
			}
			if (!isset($this->options['db']) || $this->options['db'] != $this->db) {
				global $wpdb;
	   			if ($wpdb->get_var("SHOW TABLES LIKE '$this->table'") != $this->table) {
	   				$sql = "CREATE TABLE ".$this->table." (
						  id mediumint(9) NOT NULL AUTO_INCREMENT,
						  year smallint(5) NOT NULL,
						  month tinyint(3) NOT NULL,
						  day tinyint(3) NOT NULL,
						  PRIMARY KEY (id),
						  UNIQUE KEY `date` (`year`,`month`,`day`)					  
						);";
					require_once(ABSPATH.'wp-admin/includes/upgrade.php');
					dbDelta($sql);
					$this->options['db'] = $this->db;
					update_option($this->tag, $this->options);
	   			}
			}	
		}
		function css()
		{
			global $post;
			if ($post && strstr($post->post_content, '['.$this->tag.']')) {
				wp_enqueue_style(
					$this->tag,
					WP_PLUGIN_URL.'/'.$this->tag.'/style.css',
					false,
					false,
					'screen'
				);
			}
		}
		function shortcode($atts)
		{
			extract(shortcode_atts(array(
				'display' => false
			), $atts));
			switch ($display) {
				case 'dropdown':
					return $this->dropdown();
				break;
				case 'year':
					return $this->year;
				break;
				default:					
					return $this->output('calendar.php');
				break;
			}
		}
		function dropdown($atts = array())
		{
			extract(shortcode_atts(array(
				'before' => '',
				'after' => '',
				'display' => false
			), $atts));
			$dropdown = $before.$this->output('dropdown.php').$after;
			if ($display == true) {
				echo $dropdown;
			} else {
				return $dropdown;
			}
			return null;
		}
		function booked()
		{
			global $wpdb;
			$sql = "SELECT * FROM ".$this->table.
			       " WHERE year = ".$wpdb->escape($this->year);
  			$result = $wpdb->get_results($sql);
  			$booked = array();
			if (count($result) > 0) {
				foreach ($result AS $date) {
					$booked[$date->month][$date->day] = $date->id;
				}
			}
			return $booked;
		}
		function admin_menu()
		{
			add_submenu_page(
				'edit-pages.php',
				'Manage '.$this->name,
				$this->name,
				'administrator',
				$this->tag,
				array(&$this, 'manage')
			);
		}
		function manage()
		{
			global $wpdb;
			$booked = $this->booked();
			if (isset($_POST[$this->tag.'_booked'])) {
				$updates = &$_POST[$this->tag.'_booked'];
				$bookings = array();
				foreach ($updates AS $month => $days) {
					foreach ($days AS $day => $status) {
						if (!isset($booked[$month][$day])) {
							$bookings[$month.'-'.$day] = $this->year.", ".$month.", ".$day;
							$booked[$month][$day] = true;
						}
					}
				}
				if (count($bookings) > 0) {
					$updateSQL = "INSERT IGNORE INTO ".$this->table."
					              (year, month, day)
					              VALUES (".implode("),(", $bookings).")";
					if ($wpdb->query($updateSQL)) {
						$saved = true;
					}
				}
				$remove = array();
				foreach ($booked AS $month => $days) {
					foreach ($days AS $day => $status) {
						if (!isset($updates[$month][$day]) && !isset($bookings[$month.'-'.$day])) {
							$remove[] = $booked[$month][$day];
							unset($booked[$month][$day]);
						}
					}		
				}
				if (count($remove) > 0) {
					$removeSQL = "DELETE FROM ".$this->table."
					              WHERE `id` IN (".implode(",", $remove).")";
					if ($wpdb->query($removeSQL)) {
						$saved = true;
					}
				}
			}
			include_once('manage.php');
		}
		function plugin_row_meta($links, $file)
		{
			$plugin = plugin_basename(__FILE__);
			if ($file == $plugin) {
				return array_merge(
					$links,
					array(sprintf(
						'<a href="plugins.php?page=%s">%s</a>',
						$this->tag, __('Settings')
					))
				);
			}
			return $links;
		}
		function output($file)
		{
			ob_start();
			$booked = $this->booked();
			include(basename($file));
			$content = ob_get_contents(); ob_end_clean();
			return $content;
		}
	}
	$availability = new Availability();
	if (isset($availability)) {
		function availability_dropdown($b='',$a='',$d=true) {
			return apply_filters('availability_dropdown', array('before'=>$b,'after'=>$a,'display'=>$d));
		}
	}
}
